/* Copyright (c) 2012 Sven Herrmann and Andreas Spillner

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include "polymake/client.h"
#include "polymake/Matrix.h"
#include "polymake/Graph.h"
#include "polymake/graph/MaxPlusRational.h"
#include "polymake/Set.h"
#include "polymake/linalg.h"

namespace polymake { namespace graph {

Matrix<Rational> metric_from_graph(perl::Object g,const Array<int>& elements)
{
  const Graph<>& G=g.give("ADJACENCY");
  const int d=elements.size();
  const int n=G.nodes();
  Matrix<MaxPlusRational> distances=unit_matrix<MaxPlusRational>(n);
  EdgeMap<Undirected,Rational> weights;
  if(g.lookup("EDGE_LENGTHS")>>weights)
    for (Entire< Edges<Graph<> > >::const_iterator e=entire(edges(G)); !e.at_end(); ++e)
      distances(e.from_node(),e.to_node())=distances(e.to_node(),e.from_node())=-weights[*e];
  else
   for (Entire< Edges<Graph<> > >::const_iterator e=entire(edges(G)); !e.at_end(); ++e)
      distances(e.from_node(),e.to_node())=distances(e.to_node(),e.from_node())=-1;


  const double l=std::log(n-1)/std::log(2);
  const int ll=std::ceil(l);
  for (int i=0; i<ll; ++i) {
     distances=distances*distances;
  }
  
  Matrix<Rational> M(d,d);
  for (int i=0;i<d;++i)
     for (int j=i+1;j<d;++j)
        M(i,j)=M(j,i)=-distances(elements[i],elements[j]).get_value();
  return M;
}

UserFunction4perl("# @category Other"
                  "# Computes the distances defined by the shortest path distance"
                  "# between the //labeled nodes// in //g//."
                  "# @param WeightedLabeledGraph g"
                  "# @param Array<Int> labeled nodes"
                  "# @return Matrix",
                  &metric_from_graph,"metric_from_graph"); 

} }
