/* Copyright (c) 2012 Sven Herrmann and Andreas Spillner

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include "polymake/Matrix.h"
#include "polymake/Set.h"
#include "polymake/Array.h"

namespace polymake { namespace metric {


perl::Object multipartite_graph_metric(const Array<int>& comps)
{
   int n=0;
   for (Entire<Array<int> >::const_iterator i=entire(comps);!i.at_end();++i) n+=*i;   
   Matrix<int> M=ones_matrix<int>(n,n);
   int start=0;
   for (Entire<Array<int> >::const_iterator i=entire(comps);!i.at_end();++i) {
     M.minor(sequence(start,*i),sequence(start,*i))=2*ones_matrix<int>(*i,*i);
     start+=*i;
   }
   for (int i=0; i<n; ++i) M(i,i)=0;
  perl::Object m("Metric");
  m.take("DISTANCE_MATRIX")<<M;
  return m;
}
perl::Object bipartite_graph_metric(const int n1,const int n2)
{
   Array<int> a(2);
   a[0]=n1;
   a[1]=n2;
   return multipartite_graph_metric(a);
}





UserFunction4perl("# @Category Producing from scratch"
                  "# Gives a metric obtained from a bipartite graph with component sizes n1 and n2",
                 &bipartite_graph_metric,"bipartite_graph_metric"); 

UserFunction4perl("# @Category Producing from scratch"
                  "# Gives a metric obtained from a multipartite graph where //a// gives the numbers of elements of each component",
                 &multipartite_graph_metric,"multipartite_graph_metric"); 



} }
