/* Copyright (c) 2012 Sven Herrmann and Andreas Spillner

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include "polymake/client.h"
#include "polymake/Matrix.h"

namespace polymake { namespace metric {

Matrix<int> circle_metric_matrix(const int d)
{
  Matrix<int> M(d,d);
  for (int i=0;i<d;++i)
     for (int j=i+1;j<d;++j)
        M(i,j)=M(j,i)=j-i<=d/2?j-i:d-j+i;
  return M;
}

perl::Object circle_metric(const int d)
{
  Matrix<int> M=circle_metric_matrix(d);
  perl::Object m("Metric");
  m.take("DISTANCE_MATRIX")<<M;
  return m;
}

perl::Object two_circle_metric(const int n)
{
  const int d=2*(n-1);
  const Matrix<int> circ=circle_metric_matrix(n);
  Matrix<int> M(d,d);
  for (int i=0;i<n-1;++i)
    for (int j=0;j<n-1;++j)
        M(i,j)=M(j,i)=M(i+n-1,j+n-1)=M(j+n-1,i+n-1)=circ(i+1,j+1);
  for (int i=0;i<n-1;++i)
     for (int j=n-1;j<d;++j)
        M(i,j)=M(j,i)=1+circ(0,i+1)+circ(0,j-n+2);
  perl::Object m("Metric");
  m.take("DISTANCE_MATRIX")<<M;
  return m;
}


UserFunction4perl("# Gives a metric obtained from a circle of length n",
                 &circle_metric,"circle_metric"); 

UserFunction4perl("# Gives a metric obtained from a two circle of length n connected by an edge",
                 &two_circle_metric,"two_circle_metric"); 

} }
