/* Copyright (c) 2012 Sven Herrmann and Andreas Spillner

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include "polymake/Graph.h"
#include "polymake/client.h"

namespace polymake { namespace metric {

   template <typename Scalar>
   perl::Object combination(const int keep_only,const Array<perl::Object>& gg_in)
   {
      Entire< Array<perl::Object> >::const_iterator g_in=entire(gg_in);
      perl::Object g_out(g_in->type());
      
      Graph<> G;
      EdgeMap<Undirected,Scalar> weights(G);
      Map<std::string,int> label_map;
      int n_labeled_nodes=0;
      for (;!g_in.at_end();++g_in) {  
         //cerr<<"next graph: ";
	 //cerr<<"GRAPH"<<endl;
         const Array<int> ln_map=g_in->give("LABELED_NODES_MAP");
         const Array<std::string> g_labels=g_in->give("LABELS");
	 //cerr<<g_labels<<endl;
	 const int n=ln_map.size();
         //cerr<<n<<" nodes"<<endl;
         Array<int> nodes_in_G(n);
         for (int i=0;i<n;++i) {
            if (ln_map[i]<0) nodes_in_G[i]=G.add_node();
            else {
               Entire<Map<std::string,int> >::const_iterator j=label_map.find(g_labels[ln_map[i]]);
               if (j==label_map.end()) {
	          label_map[g_labels[ln_map[i]]]= nodes_in_G[i]=G.add_node();
		  ++n_labeled_nodes;
	       }
               else nodes_in_G[i]=j->second;
               //cerr<<nodes_in_G[i]<<endl;
            }
         } 
         //cerr<<"nodes done"<<endl;  
         const Graph<> G_in=g_in->give("ADJACENCY");     //cerr<<"..."<<endl;
         EdgeMap<Undirected,Scalar> g_weights(G_in);
         //cerr<<"constructed"<<endl;
         g_in->give("EDGE_LENGTHS")>>g_weights;
         //cerr<<"..."<<endl;
         for (Entire<Edges<Graph<> > >::const_iterator e=entire(edges(G_in));!e.at_end();++e) {
             //cerr<<nodes_in_G[e.from_node()]<<"to"<<nodes_in_G[e.to_node()]<<endl;
             weights[G.edge(nodes_in_G[e.from_node()],nodes_in_G[e.to_node()])]=g_weights[*e];
         }
         //cerr<<"Graph:"<<G<<endl;
     }
     //cerr<<"done"<<endl;
     
     Array<int> labeled_nodes(n_labeled_nodes);
     
     if (keep_only<0) {
       Array<std::string> labels(n_labeled_nodes);
       int i=0;
       for(Entire<Map<std::string,int> >::const_iterator m=entire(label_map);i<n_labeled_nodes;++m,++i) {
         labels[i]=m->first;
	 labeled_nodes[i]=m->second;
       }	
       g_out.take("LABELS")<<labels;
     }
     else {
       for(Entire<Map<std::string,int> >::const_iterator m=entire(label_map);!m.at_end();++m) {
         const int number=atoi(m->first.c_str());
         if (number<keep_only) labeled_nodes[number]=m->second;
       }
     }  
     g_out.take("ADJACENCY")<<G;
     g_out.take("N_NODES")<<G.nodes();
     //cerr<<"Edges:" <<G.edges()<<endl<<weights<<"x"<<endl;
     g_out.take("EDGE_LENGTHS")<<weights;
     //cerr<<"Labels: "<<labeled_nodes<<endl;
     g_out.take("LABELED_NODES")<<labeled_nodes;

     return g_out;     
}

   template <typename Scalar>
perl::Object combination(const Array<perl::Object>& gg_in) {
  return combination<Scalar>(-1,gg_in);
}

FunctionTemplate4perl("combination<Scalar>($, WeightedLabeledGraph<Scalar> +)");


   UserFunctionTemplate4perl("# @category Other"
			     "# Combine several weighted labeled graphs and identify nodes with identical labels"
                             "# @param WeightedLabeledGraph G ... graphs to be combined"
			     "# @return WeightedLabeledGraph",
			     "combination<Scalar>(WeightedLabeledGraph<Scalar> + )");
} }
