/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: poly2lp.cc 9551 2010-03-09 14:30:03Z herr $
*/

#include "polymake/client.h"
#include "polymake/Matrix.h"
#include "polymake/Set.h"
#include "polymake/Array.h"
#include "polymake/Rational.h"
#include <fstream>

namespace polymake { namespace metric {


namespace {
  void print_metric(perl::Object m, std::ostream& os)
  {
      const int precision=16;
      const int n=m.give("N_ELEMENTS");
      Array<std::string> labels;
      os << "#nexus"<<endl<<endl
         << "BEGIN TAXA;"<<endl
           << "DIMENSIONS NTAX="<<n<<";"<<endl
           << "TAXLABELS";
      if(m.lookup("LABELS")>> labels) for (int i=0;i<n;++i) os<<" "<<labels[i];
      else for (int i=1;i<=n;++i) os<<" "<<i;
      os <<";\nEND;"<<endl<<endl;
    
      if (m.exists("DISTANCES")) m.give("DISTANCE_MATRIX");
      Matrix<double> D;
      if (m.lookup("DISTANCE_MATRIX") >>D ){
        const int n_variables=D.cols();
        os << std::setprecision(precision)
	    << "BEGIN DISTANCES;\n"
	    << "DIMENSIONS ntax="<<n_variables<<";"<<endl
            << "FORMAT TRIANGLE=both diagonal LABELS=no;"<<endl
            << "MATRIX"<<endl;
        for (int i=0;i<n_variables;++i) {
          os <<"["<<(i+1)<<"] ";
          for (int j=0;j<n_variables;++j) os << D(i,j)<<" ";
          os<<endl;
        }
        os << ";\nEND;" << endl<<endl;
      }
      Array<std::pair<Set<int>,Rational > > splits;
      if (m.lookup("SPLITS")>> splits){
        const int n_splits=splits.size();
        os << "BEGIN Splits;"<<endl
           << "DIMENSIONS ntax="<<n<<" nsplits="<<n_splits<<";"<<endl
           << "FORMAT labels=no weights=yes confidences=no intervals=no;"<<endl
           << "MATRIX"<<endl;
        for (int i=0;i<n_splits;++i) {
          const int size=splits[i].first.size();
          const int ssize=size<=n/2?size:n-size;
          os<<"["<<(i+1)<<", size="<<ssize<<"]\t"<<std::setprecision(precision)<<splits[i].second<<"\t";
          for (Entire<Set<int> >::const_iterator j=entire(splits[i].first); !j.at_end();++j) os<<" "<<*j;
          os<<","<<endl;
        }
        os <<";\nEND;"<<endl<<endl;
      }
}

}//anonymous namespace

void metric2nex(perl::Object m, const std::string& file)
{
  if (file.empty() || file=="-")
    print_metric(m,std::cout);
    else {
      std::ofstream os(file.c_str());
      print_metric(m,os);
    }
}


UserFunction4perl("# @category Utilities"
		     "# Convert a polymake metric into a nex-file."
		     "# @param Metric m"
		     "# @param String file default value: standard output",
		     &metric2nex, "metric2nex(Metric;$='')");
} }
