/* Copyright (c) 2012 Sven Herrmann and Andreas Spillner

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#include "polymake/client.h"
#include "polymake/Rational.h"
#include "polymake/SparseMatrix.h"
#include "polymake/ListMatrix.h"

namespace polymake { namespace metric {


namespace {

inline int index(const int i, const int j, const int n) {
   if (j>i) return (n-1)*i-(i*(i-1))/2+j-i-1;
   return (n-1)*j-(j*(j-1))/2+i-j-1;
}

}

perl::Object metric_cone(const int n) {
  const int n_distances=(n*(n-1))/2;
  perl::Object c("polytope::Cone<Rational>");
  if (n<=2) {
    c.take("INEQUALITIES")<<unit_matrix<int>(n_distances);
    return c;
  }
  ListMatrix<SparseVector<int> > ineqs;
  for (int i=0; i<n;++i) 
   for (int j=i+1; j<n;++j)
    for (int k=0; k<n;++k) if (k!=i&&k!=j) {
      SparseVector<int> s(n_distances);
      s[index(i,k,n)]=s[index(k,j,n)]=1;
      s[index(i,j,n)]=-1;
      ineqs/=s;
   }

  const SparseMatrix<int> m=(unit_matrix<int>(n_distances)/ineqs);
  c.take("INEQUALITIES")<<m;
  return c;
}

UserFunction4perl("# gives the metric cone",
		  &metric_cone, "metric_cone");

} }
