/* Copyright (c) 2012 Sven Herrmann and Andreas Spillner

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/
#include "polymake/client.h"
#include "polymake/Matrix.h"
#include <cmath>

namespace polymake { namespace metric {
template <typename Scalar>
perl::Object points2metric(const Matrix<Scalar> &V)
{
  const int n=V.rows();
  Matrix<Scalar> dist(n,n);
  for (int i=0; i<n; ++i)
      for (int j=i; j<n; ++j)
        dist(i,j)=dist(j,i)=accumulate(attach_operation(V.row(i)-V.row(j), operations::abs_value()), operations::add());
  perl::ObjectType t = perl::ObjectType::construct<Scalar>("Metric");
  perl::Object m(t);
  m.take("DISTANCE_MATRIX")<<dist;
  return m;
}


UserFunctionTemplate4perl("# @Category Producing from scratch"
                  "# Produces a metric obtained from the l1 (Manhattan) distances of //points//."
                  "# @param Matrix points"
                  "# @return Metric",
                   "points2metric(Matrix)");

} }

// Local Variables:
// mode:C++
// c-basic-offset:3
// indent-tabs-mode:nil
// End:
