/* Copyright (c) 2012 Sven Herrmann and Andreas Spillner
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version: http://www.gnu.org/licenses/gpl.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
import java.io.*;
import java.util.*;

public class IO
{
//************************************************************************************
//general methods: open a file etc.
//************************************************************************************

   public static PrintWriter openprintwriter(String filename)
   {
      PrintWriter pw =  null;
      try 
      {   
         pw = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
      }catch (IOException exception) {System.out.println("Error while creating nexus file");}
      return pw;
   }

   public static void closeprintwriter(PrintWriter pw)
   {
      pw.close();
   }

   public static LineNumberReader openlinereader(String filename)
   {
      LineNumberReader ln_reader = null;
      try 
      {
         ln_reader = new LineNumberReader(new FileReader(filename));
      }catch (IOException exception) {System.out.println("Error while reading nexus file.");}
      return ln_reader;
   }

   public static void closelinereader(LineNumberReader ln_reader)
   {
      try 
      {
         ln_reader.close();
      }catch (IOException exception) {System.out.println("Error while reading nexus file.");}
   }

   //The header of the nexus file
   public static void writeheader(PrintWriter pw)
   {
      pw.println("#nexus");
      pw.println(""); 
   }

//********************************************************************************************
//methods for reading input file
//********************************************************************************************

    public static TreeSet readPoints(String filename)
    {
       LineNumberReader lnr = null;
       try 
       { 
          lnr = new LineNumberReader(new FileReader(filename));
       }catch (IOException exception) {System.out.println("Error while opening input file.");}

       String line = null;
       TreeSet plist = new TreeSet(new PointLexComparatorX());
       int idx = 0;
       String x = null;
       String y = null;

       do
       {
          try
          {
             line = lnr.readLine();
          }catch (IOException exception) 
          {
             System.out.println("Error while reading point from file");
          }
          
          if((line != null) && (line.length() > 0))
	  {
             line = line.trim();
             idx = line.indexOf(' ');
             x = (line.substring(0,idx)).trim();
             y = (line.substring(idx)).trim();
             plist.add(new Point(x,y));
          }
       }while((line != null) && (line.length() > 0));

       try 
       {
          lnr.close();
       }catch (IOException exception) {System.out.println("Error while closing input file.");}

       return plist;
    }

//************************************************************************************************
//methods to print some information on the screen -- for testing purposes    
//************************************************************************************************

   public static void printPoints(TreeSet plist)
   {
      System.out.println("Points: " + plist.size());

      Iterator iter = plist.iterator();
      while(iter.hasNext())
      {
         Point p = (Point)iter.next();
         System.out.println("(" + p.x.toString() + "," + p.y.toString() + ")");
      }
   }

   public static void printPairs(TreeSet plist)
   {
      System.out.println("Pairs: " + plist.size());

      Iterator iter = plist.iterator();
      while(iter.hasNext())
      {
         Pair a = (Pair)iter.next();
         System.out.println("((" + a.p.x.toString() + "," + a.p.y.toString() + "),(" + a.q.x.toString() + "," + a.q.y.toString() + "))");
      }
   }

   public static void printEdges(LinkedList edges)
   {
      System.out.println("Edges: " + edges.size());

      Iterator iter = edges.iterator();
      while(iter.hasNext())
      {
         Edge e = (Edge)iter.next();
         System.out.println("((" + e.u.x.toString() + "," + e.u.y.toString() + "),(" + e.v.x.toString() + "," + e.v.y.toString() + ")) index: " + e.idx + " offset: " + e.offset);
      }
   }

   public static void printVertices(LinkedList vertices)
   {
      System.out.println("Vertices: " + vertices.size());

      Iterator iter = vertices.iterator();
      while(iter.hasNext())
      {
         Vertex w = (Vertex)iter.next();
         System.out.println("(" + w.p.x.toString() + "," + w.p.y.toString() + ") edges: " + w.elist.size());
      }
   }

//*************************************************************************************************
//methods for writing description of MIP to file
//*************************************************************************************************

  public static void writeMIPToFile(LinkedList edges,LinkedList vertices,TreeSet genPairs,String filename)
  {
     PrintWriter pw = openprintwriter(filename);

     //write objective function
     pw.println("minimize");

     String line = "\t";

     Iterator iter = edges.iterator();
     int cur = 1;

     while(iter.hasNext())
     {
        Edge e = (Edge)iter.next();
        if(e.idx >= 0)
        {
           if(cur == 1)
           {
              line = line + " " + e.length() + " y" + cur;
              cur++;
           }
           else
           {
              line = line + " + " + e.length() + " y" + cur;
              cur++;
           }
        }
     }
     pw.println(line);
    
     //write constraints
     pw.println("");
     pw.println("subject to");

     String ending = null;
     int eidx = 0;
     int minidx = -1;
     int maxidx = -1;
     int curcons = 1;
     iter = vertices.iterator();

     while(iter.hasNext())
     {
        Vertex w = (Vertex)iter.next();

        Iterator piter = genPairs.iterator();

        while(piter.hasNext())
        {
           Pair a = (Pair)piter.next();

           if((a.p.x).equals(w.p.x) && (a.p.y).equals(w.p.y))
           {
              ending = " = 1";
           }
           else if((a.q.x).equals(w.p.x) && (a.q.y).equals(w.p.y))
           {
              ending = " = -1";
           }
           else
           {
              ending = " = 0";
           }

           Iterator eiter = w.elist.iterator();

           line = "";
           
           while(eiter.hasNext())
           {
              Edge e = (Edge)eiter.next();

              if(e.uvpairs.contains(a))
              {
                 //System.out.println("Vertex (" + (w.p.x).toString() + "," + (w.p.y).toString() + ")" + curcons);
                 eidx = e.offset + (e.uvpairs.headSet(a)).size();
                 if((w.p.x).equals(e.u.x) && (w.p.y).equals(e.u.y))
                 {
                    line = line.concat(" + y" + eidx);
                 }
                 else
                 {
                    line = line.concat(" - y" + eidx);
                 }
                 if(minidx == -1 || minidx > eidx) {minidx = eidx;}
                 if(maxidx == -1 || maxidx < eidx) {maxidx = eidx;}
              }
              else if(e.vupairs.contains(a))
              {
                 //System.out.println("Vertex (" + (w.p.x).toString() + "," + (w.p.y).toString() + ")" + curcons);
                 eidx = e.offset + e.uvpairs.size() + (e.vupairs.headSet(a)).size();
                 if((w.p.x).equals(e.u.x) && (w.p.y).equals(e.u.y))
                 {
                    line = line.concat(" - y" + eidx);
                 }
                 else
                 {
                    line = line.concat(" + y" + eidx);
                 }
                 if(minidx == -1 || minidx > eidx) {minidx = eidx;}
                 if(maxidx == -1 || maxidx < eidx) {maxidx = eidx;}
              }
           }

           if(line.length() > 0)
           {
              line = "\tconsa" + curcons + ": " + line + ending;
              pw.println(line);
              curcons++;
           }
        }
     }

     iter = edges.iterator();
     
     while(iter.hasNext())
     {
        Edge e = (Edge)iter.next();

        int i = 0;
        
        Iterator piter = e.uvpairs.iterator();
        while(piter.hasNext())
        {
           Pair a = (Pair)piter.next();
           line = "\tconsb" + curcons + ": " + " y" + e.idx + " - y" + (e.offset+i) + " >= 0";
           pw.println(line);
           curcons++;
           i++;
        }

        piter = e.vupairs.iterator();
        while(piter.hasNext())
        {
           Pair a = (Pair)piter.next();
           line = "\tconsb" + curcons + ": " + " y" + e.idx + " - y" + (e.offset+i) + " >= 0";
           pw.println(line);
           curcons++;
           i++;
        }
     }

     //specify binary variables
     pw.println("");
     pw.println("binary");

     for(int i = 1;i <= maxidx; i++)
     {
        pw.println("\ty" + i);
     }

     pw.println("");
     pw.println("end");

     closeprintwriter(pw);
  }

  //writes list of relevant edges to file so that we have it
  //after solving the MIP
  public static void writeGridToFile(LinkedList edges,String filename)
  {
     Iterator iter = edges.iterator();

     PrintWriter pw = openprintwriter(filename);

     while(iter.hasNext())
     {
        Edge e = (Edge)iter.next();
        if(e.idx >= 0)
        {
           pw.println("((" + e.u.x.toString() + "," + e.u.y.toString() + "),(" + e.v.x.toString() + "," + e.v.y.toString() + "))");
        }
     }

     closeprintwriter(pw);
  }

//**************************************************************************
//methods for extracting the solution from output file of MIP-solver
//currently we assume the output was produced by glpsol
//**************************************************************************
  
   public static void extractEdges(String infileEdges,String infileMIPSolution,String outfile)
   {
      PrintWriter pw = openprintwriter(outfile);
      LineNumberReader lnredges = openlinereader(infileEdges);
      LineNumberReader lnrsol = openlinereader(infileMIPSolution);

      String line = null;
      int nrows = 0;
      int sol = 0;


      try
      {
         line = lnrsol.readLine();
         line = line.trim();
         line = (line.substring(0,line.indexOf(' '))).trim();
         nrows = Integer.parseInt(line);
         line = lnrsol.readLine();
         line = line.trim();
         line = (line.substring(line.indexOf(' '))).trim();
         sol = Integer.parseInt(line);
      }
      catch (IOException exception) 
      {
         System.out.println("Error while reading MIP solution");
      }
       
      pw.println("total length: " + sol);

      for(int i = 0;i<nrows;i++)
      {
         try
         {
            lnrsol.readLine();
         }
         catch (IOException exception) 
         {
            System.out.println("Error while reading MIP solution");
         }
      }

      String edgeline = null;
      String solline = null;

      do
      {
         try
         {
            solline = lnrsol.readLine();
            edgeline = lnredges.readLine();
         }
         catch (IOException exception) 
         {
            System.out.println("Error while reading MIP solution");
         }

         if((edgeline != null) && (edgeline.length() > 0) && (Integer.parseInt(solline) == 1))
         {
            pw.println(edgeline);
         }
      }while((edgeline != null) && (edgeline.length() > 0));

      closelinereader(lnredges);
      closelinereader(lnrsol);
      closeprintwriter(pw);
   }
}
