/* Copyright (c) 2012 Sven Herrmann and Andreas Spillner
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version: http://www.gnu.org/licenses/gpl.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
import java.math.*;

public class Graph
{
   public int nvert; //number of vertices
   public int nelem; //number of elements in X
   public int nedges; //number of edges
   public int[] elements; //indices of elements in X
   public Edge[] edges; //edges of the graph
   public BigInteger[][] dist; //shortest path distances between vertices

   //This method computes the shortest path distances
   public void computeShortestPathDistances()
   {
      //loop variables
      int i = 0;
      int j = 0;
      int k = 0;
      int l = 0;

      BigInteger bmax = null;

      //allocate memory for distance matrices
      dist = new BigInteger[nvert][nvert];
      BigInteger[][] tmpdist = new BigInteger[nvert][nvert];     

      //initialize distance matrix
      for(i=0;i<nedges;i++)
      {
	 dist[edges[i].u-1][edges[i].v-1] = new BigInteger((edges[i].w).toString());
         dist[edges[i].v-1][edges[i].u-1] = new BigInteger((edges[i].w).toString());
         if((bmax == null) ||
            (bmax.compareTo(edges[i].w) == -1))
         {
            bmax = edges[i].w;
         }
      }

      bmax = new BigInteger((bmax.multiply(new BigInteger(Integer.toString(nvert)))).toString());

      for(i=0;i<nvert;i++)
      {
         for(j=0;j<nvert;j++)
         {
             if(dist[i][j] == null)
	     {
                dist[i][j] = new BigInteger(bmax.toString());
             }
         }
         dist[i][i] = new BigInteger("0");
      }

      //run Floyd-Warshall algorithm
      for(k=0;k<nvert;k++)
      {
         //save current distance matrix
         for(i=0;i<nvert;i++)
         {
            for(j=0;j<nvert;j++)
            {
               tmpdist[i][j] = dist[i][j];
            }
         }
         //compute new distance matrix
         for(i=0;i<nvert;i++)
         {
            for(j=0;j<nvert;j++)
            {
               if((tmpdist[i][j]).compareTo(tmpdist[i][k].add(tmpdist[k][j])) == 1)
               {
                  dist[i][j] = (tmpdist[i][k]).add(tmpdist[k][j]);
               }
               else
               {
                  dist[i][j] = new BigInteger(tmpdist[i][j].toString());
               }
            }
         }
      }
   }

   //prints shortest path distance matrix on the screen
   public void printShortestPathDistances()
   {
      int i = 0;
      int j = 0;

      for(i=0;i<nvert;i++)
      {
         for(j=0;j<nvert;j++)
         {
            System.out.print(dist[i][j].toString() + " ");
         }
         System.out.print("\n");
      }
   }

   //check if edge e is on a shortest path from vertex x to vertex y
   public boolean isOnShortestPath(Edge e, int x, int y)
   {
      if(((dist[x-1][y-1]).compareTo((dist[x-1][e.u-1]).add(e.w.add(dist[e.v-1][y-1]))) == 0) ||
         ((dist[x-1][y-1]).compareTo((dist[x-1][e.v-1]).add(e.w.add(dist[e.u-1][y-1]))) == 0))
      {
         return true;
      }
      else
      {
         return false;
      }
   } 
}
