/* Copyright (c) 2012 Sven Herrmann and Andreas Spillner
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version: http://www.gnu.org/licenses/gpl.txt.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
import java.io.*;
import java.util.*;
import java.math.*;

public class IO
{
//************************************************************************************
//general methods: open a file etc.
//************************************************************************************

   public static PrintWriter openprintwriter(String filename)
   {
      PrintWriter pw =  null;
      try 
      {   
         pw = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
      }catch (IOException exception) {System.out.println("Error while creating file");}
      return pw;
   }

   public static void closeprintwriter(PrintWriter pw)
   {
      pw.close();
   }

   public static LineNumberReader openlinereader(String filename)
   {
      LineNumberReader ln_reader = null;
      try 
      {
         ln_reader = new LineNumberReader(new FileReader(filename));
      }catch (IOException exception) {System.out.println("Error while reading file.");}
      return ln_reader;
   }

   public static void closelinereader(LineNumberReader ln_reader)
   {
      try 
      {
         ln_reader.close();
      }catch (IOException exception) {System.out.println("Error while reading file.");}
   }

//********************************************************************************************
//methods for reading input file
//********************************************************************************************

    public static Distance readDistance(String filename, int nelem)
    {
       LineNumberReader lnr = null;
       try 
       { 
          lnr = new LineNumberReader(new FileReader(filename));
       }catch (IOException exception) {System.out.println("Error while opening input file.");}

       String line = null;    //line of file
       Distance dist = new Distance();
       dist.d = new BigInteger[nelem][nelem];

       int i = 0; //loop variable
       int j = 0; //loop variable
       int idx = 0; //position in input line

       for(i=0;i<nelem;i++)
       {
          try
          {
             line = lnr.readLine();
          }catch (IOException exception) 
          {
             System.out.println("Error while reading from file");
          }

          for(j=0;j<nelem-1;j++)
          {
             line = line.trim();
             idx = line.indexOf(' ');
             dist.d[i][j] = new BigInteger((line.substring(0,idx)).trim());
             line = (line.substring(idx)).trim();            
          }
          dist.d[i][j] = new BigInteger(line);
       }

       try 
       {
          lnr.close();
       }catch (IOException exception) {System.out.println("Error while closing input file.");}

       return dist;
    }

    public static Graph readGraph(String filename)
    {
       LineNumberReader lnr = null;
       try 
       { 
          lnr = new LineNumberReader(new FileReader(filename));
       }catch (IOException exception) {System.out.println("Error while opening input file.");}

       String line = null;    //line of file
       Graph g = new Graph(); //input graph
       int i = 0;             //loop variable
       int idx = 0;           //index used to parse edges
       

       //read number of vertices
       try
       {
          line = lnr.readLine();
       }catch (IOException exception) 
       {
          System.out.println("Error while reading from file");
       }
       if((line != null) && (line.length() > 0))
       {
          g.nvert = Integer.parseInt(line.trim());
       }

       //read number of elements in X
       try
       {
          line = lnr.readLine();
       }catch (IOException exception) 
       {
          System.out.println("Error while reading from file");
       }
       if((line != null) && (line.length() > 0))
       {
          g.nelem = Integer.parseInt(line.trim());
       }

       //read number of edges
       try
       {
          line = lnr.readLine();
       }catch (IOException exception) 
       {
          System.out.println("Error while reading from file");
       }
       if((line != null) && (line.length() > 0))
       {
          g.nedges = Integer.parseInt(line.trim());
       }

       //read indices of elements in X
       g.elements = new int[g.nelem];
       for(i=0;i<g.nelem;i++)
       {
          try
          {
             line = lnr.readLine();
          }catch (IOException exception) 
          {
             System.out.println("Error while reading from file");
          }
          if((line != null) && (line.length() > 0))
          {
             g.elements[i] = Integer.parseInt(line.trim());
          }
       }

       //read edges
       g.edges = new Edge[g.nedges];
       for(i=0;i<g.nedges;i++)
       {
          try
          {
             line = lnr.readLine();
          }catch (IOException exception) 
          {
             System.out.println("Error while reading from file");
          }
          if((line != null) && (line.length() > 0))
          {
             g.edges[i] = new Edge();
             line = line.trim();
             idx = line.indexOf(' ');
             (g.edges[i]).u = Integer.parseInt((line.substring(0,idx)).trim());
             line = (line.substring(idx)).trim();
             idx = line.indexOf(' ');
             (g.edges[i]).v = Integer.parseInt((line.substring(0,idx)).trim());
             line = (line.substring(idx)).trim();
             (g.edges[i]).w = new BigInteger(line);
          }
       }

       try 
       {
          lnr.close();
       }catch (IOException exception) {System.out.println("Error while closing input file.");}

       return g;
    }

//************************************************************************************************
//methods to print some information on the screen -- for testing purposes    
//************************************************************************************************

   public static void printGraph(Graph g)
   {
      System.out.println("Number of vertices: " + g.nvert);
      System.out.println("Number of elements in X: " + g.nelem);
      System.out.println("Number of edges: " + g.nedges);

      int i = 0;

      System.out.println("List of elements in X:");
      for(i=0;i<g.nelem;i++)
      {
         System.out.println(g.elements[i]);
      }

      System.out.println("List of edges:");
      for(i=0;i<g.nedges;i++)
      {
         System.out.println(g.edges[i].u + " " + g.edges[i].v + " " + g.edges[i].w.toString());
      }
   }

//*************************************************************************************************
//methods for writing description of MIP to file
//*************************************************************************************************

  public static void writeMIPToFile(Graph g,String filename)
  {
     int i = 0;   //loop variable
     int j = 0;   //loop variable
     int k = 0;   //loop variable
     int cur = 0; //index of current variable
     int r = 0;   //number of pairs considered so far
     int s = 0;   //current vertex
     int c = 0;   //current constraint
     int rel = 0; //number of relevant edges indicent to a vertex

     PrintWriter pw = openprintwriter(filename);

     //write objective function
     pw.println("minimize");

     String line = "\t";

     for(i=0;i<g.nedges;i++)
     {
        cur = i+1;
        if(cur == 1)
        {
           line = line + " " + (g.edges[i]).w.toString() + " y" + cur;
        }
        else
        {
           line = line + " + " + (g.edges[i]).w.toString() + " y" + cur;
        }
     }    
     pw.println(line);

     //write constraints
     pw.println("");
     pw.println("subject to");

     //loop through all pairs of distinct elements in X
     for(i=0;i<g.nelem-1;i++)
     {
        for(j=i+1;j<g.nelem;j++)
        {
           //link with binary variables
           for(k=0;k<g.nedges;k++)
           {
              //check if edge is relevant for this pair of elements in X
              if(g.isOnShortestPath(g.edges[k],g.elements[i],g.elements[j]))
              {
                 c++;
                 cur = g.nedges + (r*2*g.nedges) + (2*k) + 1;
                 line = "\t" + "cons" + c + ": " + "y" + (k+1) + " - y" + cur + " >= 0";
                 pw.println(line);
                 c++;
                 cur++;
                 line = "\t" + "cons" + c + ": " + "y" + (k+1) + " - y" + cur + " >= 0";
                 pw.println(line);
              }
           }

           //flow conservation at all vertices except i and j
           for(s=0;s<g.nvert;s++)
           {
              if(((s+1) != g.elements[i]) && ((s+1) != g.elements[j]))
              {
                 rel = 0;
                 c++;
                 line = "\t" + "cons" + c + ":";
                 for(k=0;k<g.nedges;k++)
                 {
                    if(g.isOnShortestPath(g.edges[k],g.elements[i],g.elements[j]) && (g.edges[k]).u == (s+1))
                    {
                       cur = g.nedges + (r*2*g.nedges) + (2*k) + 1;
                       line = line + " + y" + cur + " - y" + (cur+1);
                       rel++;
                    }
                    if(g.isOnShortestPath(g.edges[k],g.elements[i],g.elements[j]) && (g.edges[k]).v == (s+1))
                    {
                       cur = g.nedges + (r*2*g.nedges) + (2*k) + 1;
                       line = line + " - y" + cur + " + y" + (cur+1);
                       rel++;
                    }
                 }
                 line = line + " = 0";
                 if(rel > 0)
                 {
                    pw.println(line);
                 }
                 else
                 {
                    c--;
                 }
              }
           }

           //one unit of flow out of i
           c++;
           line = "\t" + "cons" + c + ":";
           for(k=0;k<g.nedges;k++)
           {
              if(g.isOnShortestPath(g.edges[k],g.elements[i],g.elements[j]) && (g.edges[k]).u == g.elements[i])
              {
                 cur = g.nedges + (r*2*g.nedges) + (2*k) + 1;
                 line = line + " + y" + cur + " - y" + (cur+1);
              }
              if(g.isOnShortestPath(g.edges[k],g.elements[i],g.elements[j]) && (g.edges[k]).v == g.elements[i])
              {
                 cur = g.nedges + (r*2*g.nedges) + (2*k) + 1;
                 line = line + " - y" + cur + " + y" + (cur+1);
              }
           }
           line = line + " = 1";
           pw.println(line);    

           //one unit of flow into j
           c++;
           line = "\t" + "cons" + c + ":";
           for(k=0;k<g.nedges;k++)
           {
              if(g.isOnShortestPath(g.edges[k],g.elements[i],g.elements[j]) && (g.edges[k]).u == g.elements[j])
              {
                 cur = g.nedges + (r*2*g.nedges) + (2*k) + 1;
                 line = line + " + y" + cur + " - y" + (cur+1);
              }
              if(g.isOnShortestPath(g.edges[k],g.elements[i],g.elements[j]) && (g.edges[k]).v == g.elements[j])
              {
                 cur = g.nedges + (r*2*g.nedges) + (2*k) + 1;
                 line = line + " - y" + cur + " + y" + (cur+1);
              }
           }
           line = line + " = -1";
           pw.println(line); 

           //distance between i and j is upper bound
           c++;
           line = "\t" + "cons" + c + ":";
           for(k=0;k<g.nedges;k++)
           {
              if(g.isOnShortestPath(g.edges[k],g.elements[i],g.elements[j]))
              {
                 cur = g.nedges + (r*2*g.nedges) + (2*k) + 1;
                 line = line + " + " + (g.edges[k]).w.toString() + " y" + cur + " + " + (g.edges[k]).w.toString() + " y" + (cur+1); 
              }
           }
           line = line + " <= " + g.dist[g.elements[i]-1][g.elements[j]-1].toString();
           pw.println(line);

           //consider next pair of elements in X
           r++;
        }
     }

     //specify binary variables
     pw.println("");
     pw.println("binary");

     for(i = 0;i < g.nedges; i++)
     {
        pw.println("\ty" + (i+1));
     }

     pw.println("");
     pw.println("end");

     closeprintwriter(pw);
  }

//**************************************************************************
//methods for extracting the solution from output file of MIP-solver
//currently we assume the output was produced by glpsol
//**************************************************************************
  
   public static void extractEdges(String infileEdges,String infileMIPSolution,String outfile)
   {
      PrintWriter pw = openprintwriter(outfile);
      LineNumberReader lnrsol = openlinereader(infileMIPSolution);

      Graph g = IO.readGraph(infileEdges);

      String line = null;
      int nrows = 0;
      BigInteger sol = null;
      int i = 0; //loop variable

      //read length of solution and number of rows from output file of glpk
      try
      {
         line = lnrsol.readLine();
         line = line.trim();
         line = (line.substring(0,line.indexOf(' '))).trim();
         nrows = Integer.parseInt(line);
         line = lnrsol.readLine();
         line = line.trim();
         line = (line.substring(line.indexOf(' '))).trim();
         sol = new BigInteger(line);
      }
      catch (IOException exception) 
      {
         System.out.println("Error while reading MIP solution");
      }
       
      pw.println("total length: " + sol.toString());

      //skip intermediate lines to get to start of solution vector
      for(i = 0;i<nrows;i++)
      {
         try
         {
            lnrsol.readLine();
         }
         catch (IOException exception) 
         {
            System.out.println("Error while reading MIP solution");
         }
      }

      String edgeline = null;
      String solline = null;

      for(i=0;i<g.nedges;i++)
      {
         try
         {
            solline = lnrsol.readLine();
         }
         catch (IOException exception) 
         {
            System.out.println("Error while reading MIP solution");
         }

         if(Integer.parseInt(solline) == 1)
         {
            pw.println((g.edges[i]).u + " " + (g.edges[i]).v + " " + (g.edges[i]).w);
         }
      }

      closelinereader(lnrsol);
      closeprintwriter(pw);
   }
}
